
/*
	@m-onz
	jsconsole.
	now self contained js file. just include as a script tag or inline.
	includes JSON.pruned
*/

// JSON.pruned : a function to stringify any object without overflow
// example : var json = JSON.pruned({a:'e', c:[1,2,{d:{e:42, f:'deep'}}]})
// two additional optional parameters :
//   - the maximal depth (default : 6)
//   - the maximal length of arrays (default : 50)
// GitHub : https://github.com/Canop/JSON.prune
// This is based on Douglas Crockford's code ( https://github.com/douglascrockford/JSON-js/blob/master/json2.js )

(function () {
	'use strict';

	var DEFAULT_MAX_DEPTH = 6;
	var DEFAULT_ARRAY_MAX_LENGTH = 50;
	var seen; // Same variable used for all stringifications

	Date.prototype.toPrunedJSON = Date.prototype.toJSON;
	String.prototype.toPrunedJSON = String.prototype.toJSON;

	var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
		escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
		meta = {    // table of character substitutions
			'\b': '\\b',
			'\t': '\\t',
			'\n': '\\n',
			'\f': '\\f',
			'\r': '\\r',
			'"' : '\\"',
			'\\': '\\\\'
		};

	function quote(string) {
		escapable.lastIndex = 0;
		return escapable.test(string) ? '"' + string.replace(escapable, function (a) {
			var c = meta[a];
			return typeof c === 'string'
				? c
				: '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
		}) + '"' : '"' + string + '"';
	}

	function str(key, holder, depthDecr, arrayMaxLength) {
		var i,          // The loop counter.
			k,          // The member key.
			v,          // The member value.
			length,
			partial,
			value = holder[key];
		if (value && typeof value === 'object' && typeof value.toPrunedJSON === 'function') {
			value = value.toPrunedJSON(key);
		}

		switch (typeof value) {
		case 'string':
			return quote(value);
		case 'number':
			return isFinite(value) ? String(value) : 'null';
		case 'boolean':
		case 'null':
			return String(value);
		case 'object':
			if (!value) {
				return 'null';
			}
			if (depthDecr<=0 || seen.indexOf(value)!==-1) {
				return '"-pruned-"';
			}
			seen.push(value);
			partial = [];
			if (Object.prototype.toString.apply(value) === '[object Array]') {
				length = Math.min(value.length, arrayMaxLength);
				for (i = 0; i < length; i += 1) {
					partial[i] = str(i, value, depthDecr-1, arrayMaxLength) || 'null';
				}
				v = partial.length === 0
					? '[]'
					: '[' + partial.join(',') + ']';
				return v;
			}
			for (k in value) {
				if (Object.prototype.hasOwnProperty.call(value, k)) {
					try {
						v = str(k, value, depthDecr-1, arrayMaxLength);
						if (v) partial.push(quote(k) + ':' + v);
					} catch (e) { 
						// this try/catch due to some "Accessing selectionEnd on an input element that cannot have a selection." on Chrome
					}
				}
			}
			v = partial.length === 0
				? '{}'
				: '{' + partial.join(',') + '}';
			return v;
		}
	}
	JSON.pruned = function (value, depthDecr, arrayMaxLength) {
		seen = [];
		depthDecr = depthDecr || DEFAULT_MAX_DEPTH;
		arrayMaxLength = arrayMaxLength || DEFAULT_ARRAY_MAX_LENGTH;
		return str('', {'': value}, depthDecr, arrayMaxLength);
	};
}());

function prepend(elem, text) {
	var prev = elem.innerHTML
	elem.innerHTML = `${text}\n${prev}`
}

console.log = function () {
	var args = [];
	var elem = document.getElementById("js-console");
	for (var i = 0; i < arguments.length; i++) {
		var message = arguments[i];
		if (typeof message === 'object') {
			prepend(elem, `<p class="f6 lh-copy measure">${
				JSON.stringify(
					JSON.parse(JSON.pruned(message)), null, 2)
						.replace(/\\"/g, '"')}
					</p>`);
		} else {
			prepend(elem, `<p class="f6 lh-copy measure">${message}</p>`);
		}
	}
};

window.onerror = function(errorMsg, url, lineNumber, column, errorObj) {
	var error = 'Error: ' + errorMsg + ' Script: ' + url + ' Line: ' + lineNumber
	+ ' Column: ' + column + ' StackTrace: ' +  errorObj;
	var elem = document.getElementById("js-console")
	prepend(elem, `<p class="f6 lh-copy measure">${error}</p>`);
	return false;
};

window.js_console_input = function () {
	var input = document.getElementById("js-console-input")
	try {
		var result = eval(input.value)
	} catch (e) {}
	input.value = '';
	if (result) console.log(result)
	return false
}

window.open_console = function () {
	var elem = document.getElementById("web-dev-console");
	elem.className = "pa3 show"
}

window.hide_console = function () {
	var elem = document.getElementById("web-dev-console");
	elem.className = "pa3"
}

window.clear_console = function () {
	var elem = document.getElementById("js-console")
	elem.innerHTML = "";
}

// using setTimeout instead of document.onready ---> mainly because setTimeout is more consistent

setTimeout(function () {
	
	var jsconsole = document.createElement("div");

	jsconsole.innerHTML = `
	
	<style>
	
	#web-dev-console {
		font-family: "Courier New", Courier, monospace; 
		position: fixed;
		left: 50px;
		top: 0;
		background: whitesmoke;
		width: 500px; 
		height: 440px;
		box-shadow: 0 20px 30px rgba(0, 0, 0, 0.1);
		display: none;
		z-index: 111;
	}

	.show {
		display: block!important;
	}

	#js-console {
		width: 100%; 
		height: 200px; 
		overflow-y: scroll; 
		background: white; 
		color: black; 
		font-weight: 800;
	}

	#js-console-input {
		width: 381px;
	}

	#web-dev-console-label {
		font-family: "Courier New", Courier, monospace;
		box-shadow: 0 20px 30px rgba(0, 0, 0, 0.2);
		background: gray;
		position: fixed;
		bottom: 40px;
		left: 0px;
		z-index: 111;
	}

	</style>
	
	<div id="web-dev-console-label" class="pa1">
		<a href="#!" class="fr f6 link dim white db" onclick="open_console()">web tools: jsconsole </a>
	</div>
	<div id="web-dev-console" class="pa3">
		<h5>web tool: js console</h5> 
		<form class="black-80" onsubmit="return window.js_console_input()" method="POST">
			<input type="text" name="js-console-input" id="js-console-input" class="fl db border-box hover-black w-100 measure ba b--black-20 pa2 br1 mb2" />
			<input id="console-submit" type="submit" value="submit" class="f6 link dim ph3 pv2 mb2 dib white bg-black" />
		</form>
		<h5>output</h5>
		<div id="js-console" class="pa3"></div>
		<br /><hr>
		<a href="#!" class="fl f6 link dim gray db" onclick="clear_console()">clear the console</a>
		<a href="#!" class="fr f6 link dim black db" onclick="hide_console()">Hide the console</a>
	</div>`; 
	
	document.body.appendChild(jsconsole)
	
}, 1000)